%% Figure 3: Uncontrolled

iter = 15;
W = 0.1;
L = 0.5;
w = sqrt(9.81/0.4);
T = 0.5;

t = linspace(0, T, 101);

ux = 0;
uz = 0;
Xstar = -L/2;
Zstar = W/2;
Vxstar = L*w*sinh(w*T)/(2*cosh(w*T)-2);
Vzstar = -W*w*sinh(w*T)/(2*cosh(w*T)+2);
xn = Xstar;
zn = Zstar;
vxn = Vxstar;
vzn = Vzstar;
x = xn;
z = zn;
uxlist = ux;
uzlist = uz;

for i = 1:iter
    xtemp = ux + cosh(w*t) * (xn-ux) + 1/w * sinh(w*t) * vxn;
    ztemp = uz + cosh(w*t) * (zn-uz) + 1/w * sinh(w*t) * vzn;
    vxn = w*sinh(w*T) * (xn - ux) + cosh(w*T) * vxn;
    vzn = w*sinh(w*T) * (zn - uz) + cosh(w*T) * vzn;
    xn = xtemp(end);
    zn = ztemp(end);
    ux = ux + L;
    uz = uz + W * (-1)^(i+1);
    x = [x xtemp(1:end-1)];
    z = [z ztemp(1:end-1)];
    uxlist = [uxlist ux];
    uzlist = [uzlist uz];
end

figure(3)
plot(z, x)
hold on
plot(uzlist(1:2:end), uxlist(1:2:end), 'r*')
hold off
hold on
plot(uzlist(2:2:end), uxlist(2:2:end), 'b*')
hold off
set(gcf, 'defaultTextInterpreter', 'latex')
xlabel('z (m)')
ylabel('x (m)')
legend('COM Trajectory', 'Left Heel-Strikes', 'Right Heel-Strikes')

%% Figure 4: Constant Offset w/ multiple reasonable ICs

iter = 20;

bx = L/(exp(w*T)-1);
bz = W/(exp(w*T)+1);

Xstar = -bx/2 * (exp(w*T)-1);
Vxstar = bx*w/2 * (exp(w*T)+1);

Zstar = bz/2 * (exp(w*T)+1);
Vzstar = -bz*w/2 * (exp(w*T)-1);
try
clf(4)
catch
end
for j = [-0.005:0.005:0.005, 0]
    for k = [-0.05:0.05:0.05, 0]

xn = Xstar + k;
zn = Zstar + j;
vxn = Vxstar;
vzn = Vzstar;
x = xn;
z = zn;
ux = 0;
uz = 0;
uxlist = ux;
uzlist = uz;

for i = 1:iter
    xtemp = ux + cosh(w*t) * (xn-ux) + 1/w * sinh(w*t) * vxn;
    ztemp = uz + cosh(w*t) * (zn-uz) + 1/w * sinh(w*t) * vzn;
    vxn = w*sinh(w*T) * (xn - ux) + cosh(w*T) * vxn;
    vzn = w*sinh(w*T) * (zn - uz) + cosh(w*T) * vzn;
    xn = xtemp(end);
    zn = ztemp(end);
    ux = xn + vxn/w - bx;
    uz = zn + vzn/w + bz * (-1)^(i+1);
    x = [x xtemp(1:end-1)];
    z = [z ztemp(1:end-1)];
    uxlist = [uxlist ux];
    uzlist = [uzlist uz];
end

    if j == 0 && k == 0
    figure(4)
    hold on
    fixedtraj = plot(z, x, 'b');
    hold off
    hold on
    fixedsteps = plot(uzlist, uxlist, 'r*');
    hold off
    else
    figure(4)
    hold on
    ghost = plot(z, x, 'Color', [.7 .7 .7]);
    hold off
    end

    end
end
set(gcf, 'defaultTextInterpreter', 'latex')
xlabel('z (m)')
ylabel('x (m)')
fixedtraj.DisplayName = 'Fixed Point Trajectory';
fixedsteps.DisplayName = 'Heel-Strike Locations';
ghost.DisplayName = 'Perturbed Trajectories';
legend([fixedtraj, fixedsteps, ghost])

%% Figure 5: Constant Offset w/ 1 bad IC

iter = 20;

bx = L/(exp(w*T)-1);
bz = W/(exp(w*T)+1);
xn = Xstar - 0.3;
zn = Zstar - 0.1;
vxn = Vxstar;
vzn = -Vzstar;
x = xn;
z = zn;
ux = 0;
uz = 0;
uxlist = ux;
uzlist = uz;

for i = 1:iter
    xtemp = ux + cosh(w*t) * (xn-ux) + 1/w * sinh(w*t) * vxn;
    ztemp = uz + cosh(w*t) * (zn-uz) + 1/w * sinh(w*t) * vzn;
    vxn = w*sinh(w*T) * (xn - ux) + cosh(w*T) * vxn;
    vzn = w*sinh(w*T) * (zn - uz) + cosh(w*T) * vzn;
    xn = xtemp(end);
    zn = ztemp(end);
    ux = xn + vxn/w - bx;
    uz = zn + vzn/w + bz * (-1)^(i+1);
    x = [x xtemp(1:end-1)];
    z = [z ztemp(1:end-1)];
    uxlist = [uxlist ux];
    uzlist = [uzlist uz];
end

figure(5)
plot(z, x)
hold on
plot(uzlist(1:2:end), uxlist(1:2:end), 'r*')
hold off
hold on
plot(uzlist(2:2:end), uxlist(2:2:end), 'b*')
hold off
set(gcf, 'defaultTextInterpreter', 'latex')
xlabel('z (m)')
ylabel('x (m)')
legend('COM Trajectory', 'Left Heel-Strikes', 'Right Heel Strikes')

%% Figure 6: Stable Proportional Control w/ multiple IC


iter = 14;
k = 1.1/(exp(w*T) - 1);

bz = W/(exp(w*T)+1) * (1 - k/2 * (exp(w*T) - 1));
const = bz/(k - k * exp(w*T) + 2);

zstar = const * (exp(w*T) + 1);
vzstar = -const*w*(exp(w*T) - 1);
zcstar = zstar;
zc0 = zstar ;

try
clf(6)
catch
end
for j = [-0.005:0.0025:0.005, 0]
    for l = [-0.05:0.025:0.05, 0]

xn = Xstar;
zn = zstar + j;
vxn = Vxstar;
vzn = vzstar + l;
x = xn;
z = zn;
ux = 0;
uz = 0;
uxlist = ux;
uzlist = uz;

for i = 1:iter
    xtemp = ux + cosh(w*t) * (xn-ux) + 1/w * sinh(w*t) * vxn;
    ztemp = uz + cosh(w*t) * (zn-uz) + 1/w * sinh(w*t) * vzn;
    vxn = w*sinh(w*T) * (xn - ux) + cosh(w*T) * vxn;
    vzn = w*sinh(w*T) * (zn - uz) + cosh(w*T) * vzn;
    xn = xtemp(end);
    zn = ztemp(end);
    ux = xn + vxn/w - bx;
    uz = zn + vzn/w + bz * (-1)^(i+1) + k * (zn + vzn/w - zc0);
    x = [x xtemp(1:end-1)];
    z = [z ztemp(1:end-1)];
    uxlist = [uxlist ux];
    uzlist = [uzlist uz];
end

    if j == 0 && l == 0
    figure(6)
    hold on
    fixedtraj = plot(z, x, 'b');
    hold off
    hold on
    fixedsteps = plot(uzlist, uxlist, 'r*');
    hold off
    else
    figure(6)
    hold on
    ghost = plot(z, x, 'Color', [.7 .7 .7]);
    hold off
    end

    end
end

set(gcf, 'defaultTextInterpreter', 'latex')
xlabel('z (m)')
ylabel('x (m)')
fixedtraj.DisplayName = 'Fixed Point Trajectory';
fixedsteps.DisplayName = 'Heel-Strike Locations';
ghost.DisplayName = 'Perturbed Trajectories';
legend([fixedtraj, fixedsteps, ghost])

%% Figure 7: Unstable Proportional Control w/ multiple IC

iter = 10;
k = 2.1/(exp(w*T) - 1);

bz = W/(exp(w*T)+1) * (1 - k/2 * (exp(w*T) - 1));
const = bz/(k - k * exp(w*T) + 2);

zstar = const * (exp(w*T) + 1);
vzstar = -const*w*(exp(w*T) - 1);
zcstar = zstar;
zc0 = zstar ;

try
clf(7)
catch
end

for j = [-0.005:0.0025:0.005, 0]
    for l = [-0.05:0.025:0.05, 0]

xn = Xstar + 0.001;
zn = zstar + j;
vxn = Vxstar;
vzn = vzstar + l;
x = xn;
z = zn;
ux = 0;
uz = 0;
uxlist = ux;
uzlist = uz;

for i = 1:iter
    xtemp = ux + cosh(w*t) * (xn-ux) + 1/w * sinh(w*t) * vxn;
    ztemp = uz + cosh(w*t) * (zn-uz) + 1/w * sinh(w*t) * vzn;
    vxn = w*sinh(w*T) * (xn - ux) + cosh(w*T) * vxn;
    vzn = w*sinh(w*T) * (zn - uz) + cosh(w*T) * vzn;
    xn = xtemp(end);
    zn = ztemp(end);
    ux = xn + vxn/w - bx;
    uz = zn + vzn/w + bz * (-1)^(i+1) + k * (zn + vzn/w - zc0);
    x = [x xtemp(1:end-1)];
    z = [z ztemp(1:end-1)];
    uxlist = [uxlist ux];
    uzlist = [uzlist uz];
end

    if j == 0  && l == 0
    figure(7)
    hold on
    fixedtraj = plot(z, x, 'b');
    hold off
    hold on
    fixedsteps = plot(uzlist, uxlist, 'r*');
    hold off
    else
    figure(7)
    hold on
    ghost = plot(z, x, 'Color', [.7 .7 .7]);
    hold off
    end

    end
end

set(gcf, 'defaultTextInterpreter', 'latex')
xlabel('z (m)')
ylabel('x (m)')
fixedtraj.DisplayName = 'Fixed Point Trajectory';
fixedsteps.DisplayName = 'Heel-Strike Locations';
ghost.DisplayName = 'Perturbed Trajectories';
legend([fixedtraj, fixedsteps, ghost])



%% Figure 8: Prop Eigenvalues w/ varied k

% Stability Script

%% Figure 9: 1-step & 5-step constant offset basins

%% Figure 10: Constant offset basins w/ varying bz, Lmax

%% Track motion through poincare section

low = [];
high = [];
L = 0.8;
zcrange = -1:0.001:1;

for zc = zcrange
w = sqrt(9.81/0.4);
T = 0.5;
W = 0.1;

k = 0.5/(exp(w*T) - 1);
bif = (exp(w*T)*(k - exp(-w*T)*(k+1)));
bz = W/((exp(w*T)+1)/(1 - k/2*(exp(w*T) - 1)));
iter = 50;

F0 = k/(k+1) * exp(-w*T);
G0 = -bz/(k+1) * exp(-w*T);
H0 = (L-bz)/(k+1) * exp(-w*T);


B = [];
C = [];
G = [];
H = [];
G(1) = G0;
H(1) = H0;
B(1) = F0 * zc + G0;
C(1) = F0 * zc + H0;

if iter > 1
Fn = exp(-w*T);
G1 = (bz * (F0 - 1) + G0)/(exp(w*T)*(k - F0*(k+1)));
H1 = (bz * (F0 - 1) + H0)/(exp(w*T)*(k - F0*(k+1)));

B(2) = Fn * zc + G1;
C(2) = Fn * zc + H1;
G(2) = G1;
H(2) = H1;

Gtemp = G1;
Htemp = H1;
end

if iter > 2
for i = 1:iter-1

Gtemp = (bz * (Fn - 1) + Gtemp)/(exp(w*T) * (k - Fn *(k+1)));
Htemp = (bz * (Fn - 1) + Htemp)/(exp(w*T) * (k - Fn *(k+1)));
G(i+2) = Gtemp;
H(i+2) = Htemp;
Btemp = Fn * zc + Gtemp;
Ctemp = Fn * zc + Htemp;

if Btemp < Ctemp
B(i + 2) = Btemp;
C(i + 2) = Ctemp;
elseif Ctemp < Btemp
B(i + 2) = Ctemp;
C(i + 2) = Btemp;
end
end
end

low = [low max(B)];
high = [high min(C)];

end

[statemap, wlist, inertmap, coplist] = propsim([0.06, 0, 0.3], [T, W, k], 20);

zcmap = statemap(:, 3);
zcom = statemap(:, 1) + statemap(:, 2)/w;

figure(883)
plot(zstar + vzstar/w, zcstar, 'k.', 'DisplayName', 'Fixed Point', 'MarkerSize', 10);
hold on
plot(zcom(1:5), zcmap(1:5), 'r.')
hold off
hold on
plot(low(low < high), zcrange(low < high), 'b', high(low < high), zcrange(low < high), 'b', 'LineWidth', 1)
hold off


%% 3D Viability Basin Projected into 2D

low = [];
high = [];
L = 0.8;
zcrange = -1:0.001:1;

for zc = zcrange
w = sqrt(9.81/0.4);
T = 0.5;
W = 0.1;

k = 0.5/(exp(w*T) - 1);
bif = (exp(w*T)*(k - exp(-w*T)*(k+1)));
bz = W/((exp(w*T)+1)/(1 - k/2*(exp(w*T) - 1)));
iter = 50;

F0 = k/(k+1) * exp(-w*T);
G0 = -bz/(k+1) * exp(-w*T);
H0 = (L-bz)/(k+1) * exp(-w*T);


B = [];
C = [];
G = [];
H = [];
G(1) = G0;
H(1) = H0;
B(1) = F0 * zc + G0;
C(1) = F0 * zc + H0;

if iter > 1
Fn = exp(-w*T);
G1 = (bz * (F0 - 1) + G0)/(exp(w*T)*(k - F0*(k+1)));
H1 = (bz * (F0 - 1) + H0)/(exp(w*T)*(k - F0*(k+1)));

B(2) = Fn * zc + G1;
C(2) = Fn * zc + H1;
G(2) = G1;
H(2) = H1;

Gtemp = G1;
Htemp = H1;
end

if iter > 2
for i = 1:iter-1

Gtemp = (bz * (Fn - 1) + Gtemp)/(exp(w*T) * (k - Fn *(k+1)));
Htemp = (bz * (Fn - 1) + Htemp)/(exp(w*T) * (k - Fn *(k+1)));
G(i+2) = Gtemp;
H(i+2) = Htemp;
Btemp = Fn * zc + Gtemp;
Ctemp = Fn * zc + Htemp;

if Btemp < Ctemp
B(i + 2) = Btemp;
C(i + 2) = Ctemp;
elseif Ctemp < Btemp
B(i + 2) = Ctemp;
C(i + 2) = Btemp;
end
end
end

low = [low max(B)];
high = [high min(C)];

end

figure(885)
clf(885)

zcomrange = -0.01:0.0001:0.07;

[conv, div] = propshot([-0.01, 0.07, 0.0003, -0.6, 1, 0.005], [T W L k], 4);

convlength = length(conv);
divlength = length(div);

% conv = zeros(length(zcrange) * length(zcomrange), 2);
% div = zeros(length(zcrange) * length(zcomrange), 2);
% 
% convlength = 0;
% divlength = 0;
% 
% for i = 1:length(zcrange)
%     for j = 1:length(zcomrange)
%         if zcomrange(j) > low(i) && zcomrange(j) < high(i)
%             convlength = convlength + 1;
%             conv(convlength, :) = [zcomrange(j), zcrange(i)];
%         else
%             divlength = divlength + 1;
%             div(divlength, :) = [zcomrange(j), zcrange(i)];
%         end
% 
%     end
% end
ax = axes();
hold on
h(1) = plot(conv(1:convlength, 1), conv(1:convlength, 2), 'g.', 'Marker', 'square', 'MarkerFaceColor', 'g', 'MarkerSize', 1, 'DisplayName', 'Viable');
hold off
hold on
h(2) = plot(div(1:divlength, 1), div(1:divlength, 2), 'r.', 'Marker', 'square', 'MarkerFaceColor', 'r', 'MarkerSize', 1, 'DisplayName', 'Not Viable');
hold off
hold on
h(3) = plot(zstar + vzstar/w, zcstar, 'k.', 'DisplayName', 'Fixed Point', 'MarkerSize', 10);
hold off
hold on
plot(low(low < high), zcrange(low < high), 'b', high(low < high), zcrange(low < high), 'b', 'LineWidth', 1)
hold off
set(gcf, 'defaultTextInterpreter', 'latex')
xlabel('Extrapolated Center of Mass $\zeta = z + \frac{\dot{z}}{\omega}$')
ylabel('Tracking of Position Target $z_c$')

hcopy = copyobj(h, ax);
set(hcopy, 'XData', NaN, 'YData', NaN)
hcopy(1).MarkerSize = 6;
hcopy(2).MarkerSize = 6;
hcopy(3).MarkerSize = 10;


legend(hcopy, 'Location', 'southeast')

%% Defunct

L = 0.8;

figure(884)
clf(884)

    
w = sqrt(9.81/0.4);
T = 0.5;
W = 0.1;
ktwack = 2;
for kt = [0.3:0.3:1.9, 1.2, ktwack]
 low = [];
high = [];
for zc = -1:0.001:1

k = kt/(exp(w*T) - 1);
bif = (exp(w*T)*(k - exp(-w*T)*(k+1)));
bz = W/((exp(w*T)+1)/(1 - k/2*(exp(w*T) - 1)));
iter = 50;

F0 = k/(k+1) * exp(-w*T);
G0 = -bz/(k+1) * exp(-w*T);
H0 = (L-bz)/(k+1) * exp(-w*T);


B = [];
C = [];
G = [];
H = [];
G(1) = G0;
H(1) = H0;
B(1) = F0 * zc + G0;
C(1) = F0 * zc + H0;

if iter > 1
Fn = exp(-w*T);
G1 = (bz * (F0 - 1) + G0)/(exp(w*T)*(k - F0*(k+1)));
H1 = (bz * (F0 - 1) + H0)/(exp(w*T)*(k - F0*(k+1)));

B(2) = Fn * zc + G1;
C(2) = Fn * zc + H1;
G(2) = G1;
H(2) = H1;

Gtemp = G1;
Htemp = H1;
end

if iter > 2
for i = 1:iter-1

Gtemp = (bz * (Fn - 1) + Gtemp)/(exp(w*T) * (k - Fn *(k+1)));
Htemp = (bz * (Fn - 1) + Htemp)/(exp(w*T) * (k - Fn *(k+1)));
G(i+2) = Gtemp;
H(i+2) = Htemp;
Btemp = Fn * zc + Gtemp;
Ctemp = Fn * zc + Htemp;

if Btemp < Ctemp
B(i + 2) = Btemp;
C(i + 2) = Ctemp;
elseif Ctemp < Btemp
B(i + 2) = Ctemp;
C(i + 2) = Btemp;
end
end
end

low = [low max(B)];
high = [high min(C)];

end

zcrange = -1:0.001:1;

if kt == 1.2
plot(low(low < high), zcrange(low < high), 'b', high(low < high), zcrange(low < high), 'b')
hold all
elseif kt == ktwack
plot(low(low < high), zcrange(low < high), 'g', high(low < high), zcrange(low < high), 'r')
hold all
else
plot(low(low < high), zcrange(low < high), 'Color', [.7 .7 .7])
hold all
plot(high(low < high), zcrange(low < high), 'Color', [.7 .7 .7])
hold all
end

end

plot(zstar + vzstar/w, zcstar, 'k.')