%% Built to assess the stability properties of the Constant Step Width and Length Map vs. the Controller


%% Setup

W = 0.1;
S = 0.5;

w = sqrt(9.81/0.4);

t = (0.1:0.1:10)/w;
%T = 0.5;
Hofeigens = [];
Consteigens = [];
Propeigens = [];

try
clf(70)
catch
end
figure(70)

for T = t

%% IPHofSim: Controller

wT = w*T;
hiter = 1;

bx = S/(exp(w*T)-1);
bz = W/(exp(w*T)+1);

Xstar = -bx/2 * (exp(w*T)-1);
Vxstar = bx*w/2 * (exp(w*T)+1);

Zstar = bz/2 * (exp(w*T)+1);
Vzstar = -bz*w/2 * (exp(w*T)-1);

xpert = 0.01;
zpert = 0.001;
vxpert = 0.01;
vzpert = 0.001;

% First Perturbation: X
X = Xstar + xpert;

statemap1 = simplesim([X, Zstar, Vxstar, Vzstar], [T, W, S], hiter);

X = Xstar - xpert;

statemap2 = simplesim([X, Zstar, Vxstar, Vzstar], [T, W, S], hiter);

XJac = (statemap1(end, :) - statemap2(end, :))/(2*xpert);

% Second Perturbation: Z
Z = Zstar + zpert;

statemap1 = simplesim([Xstar, Z, Vxstar, Vzstar], [T, W, S], hiter);

Z = Zstar - zpert;

statemap2 = simplesim([Xstar, Z, Vxstar, Vzstar], [T, W, S], hiter);

ZJac = (statemap1(end, :) - statemap2(end, :))/(2*zpert);

% Third Perturbation: Vx
Vx = Vxstar + vxpert;

statemap1 = simplesim([Xstar, Zstar, Vx, Vzstar], [T, W, S], hiter);

Vx = Vxstar - vxpert;

statemap2 = simplesim([Xstar, Zstar, Vx, Vzstar], [T, W, S], hiter);

VxJac = (statemap1(end, :) - statemap2(end, :))/(2*vxpert);

% Fourth Perturbation: Vz
Vz = Vzstar + vzpert;

statemap1 = simplesim([Xstar, Zstar, Vxstar, Vz], [T, W, S], hiter);

Vz = Vzstar - vzpert;

statemap2 = simplesim([Xstar, Zstar, Vxstar, Vz], [T, W, S], hiter);

VzJac = (statemap1(end, :) - statemap2(end, :))/(2*vzpert);

HofJac = [XJac; ZJac; VxJac; VzJac];

Hofeigen = eigs(HofJac);
HofStable = max(Hofeigen) < 1;

Hofexp = real(log(eigs(HofJac)));
hold on
plot(w*[T, T, T, T], Hofeigen, 'b*')
hold off

Hofeigens = [Hofeigens, Hofeigen];

%% Constant

citer = 1;

Xstar = -S/2;
Zstar = W/2;
Vxstar = S*w*sinh(w*T)/(2*cosh(w*T)-2);
Vzstar = -W*w*sinh(w*T)/(2*cosh(w*T)+2);

xpert = 0.01;
zpert = 0.001;
vxpert = 0.01;
vzpert = 0.001;

% First Perturbation: X
X = Xstar + xpert;

[~, statemap1] = IPConstSim([X, Zstar, Vxstar, Vzstar], [T, W, S], citer);

X = Xstar - xpert;

[~, statemap2] = IPConstSim([X, Zstar, Vxstar, Vzstar], [T, W, S], citer);

XJac = (statemap1(end, :) - statemap2(end, :))/(2*xpert);

% Second Perturbation: Z
Z = Zstar + zpert;

[~, statemap1] = IPConstSim([Xstar, Z, Vxstar, Vzstar], [T, W, S], citer);

Z = Zstar - zpert;

[~, statemap2] = IPConstSim([Xstar, Z, Vxstar, Vzstar], [T, W, S], citer);

ZJac = (statemap1(end, :) - statemap2(end, :))/(2*zpert);

% Third Perturbation: Vx
Vx = Vxstar + vxpert;

[~, statemap1] = IPConstSim([Xstar, Zstar, Vx, Vzstar], [T, W, S], citer);

Vx = Vxstar - vxpert;

[~, statemap2] = IPConstSim([Xstar, Zstar, Vx, Vzstar], [T, W, S], citer);

VxJac = (statemap1(end, :) - statemap2(end, :))/(2*vxpert);

% Fourth Perturbation: Z
Vz = Vzstar + vzpert;

[~, statemap1] = IPConstSim([Xstar, Zstar, Vxstar, Vz], [T, W, S], citer);

Vz = Vzstar - vzpert;

[~, statemap2] = IPConstSim([Xstar, Zstar, Vxstar, Vz], [T, W, S], citer);

VzJac = (statemap1(end, :) - statemap2(end, :))/(2*vzpert);

ConstJac = [XJac; ZJac; VxJac; VzJac];
ConstJac(:, 5) = [];

Consteigen = eigs(ConstJac);
ConstStable = max(Consteigen) < 1;

Constexp = real(log(eigs(ConstJac)));
hold on
plot(w*[T, T, T, T], Consteigen, 'r.')
hold off

Consteigens = [Consteigens, Consteigen];

end

ylim([-2 2])
set(gcf, 'DefaultTextInterpreter', 'latex')
legend('Hof Controller', ['Constant \Delta', 'u'], 'Proportional Controller', 'Location', 'northeast')
xlabel('$\omega T$')
ylabel('Stability Multiplier $\lambda$')
title('Stability for Hof Controllers')

%% ConstP
figure(80)
for k = [0.001 0.01 0.1 1 10]
for T = t

wT = w*T;
piter = 1;
bz = W/(exp(w*T)+1) * (1 - k/2 * (exp(w*T) - 1));
const = bz/(k - k * exp(w*T) + 2);

zstar = const * (exp(w*T) + 1);
vzstar = -const*w*(exp(w*T) - 1);
zcstar = zstar;

zpert = 0.001;
vzpert = 0.001;
zcpert = 0.001;

% First Perturbation: Z

statemap1 = propsim([zstar + zpert, vzstar, zcstar], [T, W, k], piter);
statemap2 = propsim([zstar - zpert, vzstar, zcstar], [T, W, k], piter);

zJac = (statemap1(end,:) - statemap2(end,:))/(2*zpert);

% First Perturbation: VZ

statemap1 = propsim([zstar, vzstar + vzpert, zcstar], [T, W, k], piter);
statemap2 = propsim([zstar, vzstar - zpert, zcstar], [T, W, k], piter);

vzJac = (statemap1(end,:) - statemap2(end,:))/(2*vzpert);

% First Perturbation: Zc

statemap1 = propsim([zstar, vzstar, zcstar + zcpert], [T, W, k], piter);
statemap2 = propsim([zstar, vzstar, zcstar - zcpert], [T, W, k], piter);

zcJac = (statemap1(end,:) - statemap2(end,:))/(2*zcpert);

propJac = [zJac; vzJac; zcJac];
propeigen = eigs(propJac);


Propeigens = [Propeigens, propeigen];

end
end

xrange = [0, w*t];
yrange1 = 0.001 * (exp(xrange) - 1) - 1;
yrange2 = 0.01 * (exp(xrange) - 1) - 1;
yrange3 = 0.1 * (exp(xrange) - 1) - 1;
yrange4 = 1 * (exp(xrange) - 1) - 1;
yrange5 = 10 * (exp(xrange) - 1) - 1;

k1 = plot(w*t, Propeigens(1, 1:100), 'k.', w*t, Propeigens(2, 1:100), 'k.');
hold on
k2 = plot(w*t, Propeigens(1, 101:200), 'r.', w*t, Propeigens(2, 101:200), 'r.');
hold off
hold on
k3 = plot(w*t, Propeigens(1, 201:300), 'g.', w*t, Propeigens(2, 201:300), 'g.');
hold off
hold on
k4 = plot(w*t, Propeigens(1, 301:400), 'b.', w*t, Propeigens(2, 301:400), 'b.');
hold off
hold on
k5 = plot(w*t, Propeigens(1, 401:500), 'm.', w*t, Propeigens(2, 401:500), 'm.');
hold off
hold on
plot(xrange, yrange1, 'k', xrange, yrange2, 'r', xrange, yrange3, 'g', xrange, yrange4, 'b', xrange, yrange5, 'm')
hold off
ylim([-1 1])
set(gcf, 'DefaultTextInterpreter', 'latex')
%legend('Location', 'northeast')
legend('k = 0.001', '', 'k = 0.01', '', 'k = 0.1', '', 'k = 1', '', 'k = 10')
xlabel('$\omega T$')
ylabel('Stability Multiplier $\lambda$')



%% Inertial Plane

T = 0.5;
wT = w*T;

bx = S/(exp(w*T)-1);
bz = W/(exp(w*T)+1);

Xstar = -bx/2 * (exp(w*T)-1);
Vxstar = bx*w/2 * (exp(w*T)+1);

Zstar = bz/2 * (exp(w*T)+1);
Vzstar = -bz*w/2 * (exp(w*T)-1);

[statemap3, inertmap1] = simplesim([Xstar, Zstar, Vxstar, Vzstar], [T, W, S], 4);


[statemap4, inertmap2] = simplesim([Xstar, Zstar, Vxstar, Vzstar] + 0.001 * randn(1, 4), [T, W, S], 4);


figure(71)
plot(inertmap1(:, 2), inertmap1(:, 1), 'b')
hold on
plot(inertmap1(:, 6), inertmap1(:, 5), 'k*')
hold off
hold on
plot(inertmap2(:, 2), inertmap2(:, 1), 'r')
hold off
hold on
plot(inertmap2(:, 6), inertmap2(:, 5), 'm*')
hold off

legend('Steady State', 'SS Steps', 'Perturbed', 'Pert Steps')
