function [conv, div] = scattershot(staterange, Param, iter)

T = Param(1);
W = Param(2);
L = Param(3); % Max Distance
w = sqrt(9.81/(L/2));

zmin = staterange(1);
zmax = staterange(2);
zstep = staterange(3);
vzmin = staterange(4);
vzmax = staterange(5);
vzstep = staterange(6);

bz = W/(exp(w*T)+1);
A = [cosh(w*T) sinh(w*T)/w; w*sinh(w*T) cosh(w*T)];
B = [0 1/w; 0 -1];

Zstar = bz/2 * (exp(w*T)+1);
Vzstar = -bz*w/2 * (exp(w*T)-1);

conv = [];
div = [];

for j = zmin:zstep:zmax

    for k = vzmin:vzstep:vzmax

        zprev = [j; k];
        zstate = [j; k];
        flag = true;
        convflag = true;
        convnumber = 1;
        i = 1;

        while i <= iter && flag == true

            zprev = zstate;
            zstate = A * zstate;

            if zstate(1) + zstate(2)/w < -bz || zstate(1) + zstate(2)/w > L-bz

                flag = false;
                div = [div; j, k];

            end

            zstate = B*zstate + [bz; 0];
            
            zdist = norm(zstate - [Zstar; Vzstar]);

            if convflag && zdist < 1e-6
                convnumber = i;
                convflag = false;
            end

            i = i + 1;

        end

        if flag == true

            conv = [conv; j, k, convnumber];
            %conv = [conv; j, k];

        end

        flag = false;
        
    end

end
   
try
    clf(2);
catch
end

figure(2)

scatter(conv(:,1), conv(:,2), 'g.')%, 3, conv(:,3))
%colormap winter
%imagesc('XData', conv(:, 1), 'YData', conv(:, 2), 'CData', conv(:, 3));
%colormap winter
%plot(conv(:, 1), conv(:, 2), 'g.')
hold on
plot(div(:, 1), div(:, 2), 'r.')
hold off
set(gcf, 'defaultTextInterpreter', 'latex')
xlabel('Initial Lateral Position $z$ (m)')
ylabel('Initial Lateral Velocity $\dot{z}$ (m/s)')
legend('Converge', 'Diverge')
%title(['Convergence for $L_{max}$ = ', num2str(L)]);
%grad = colorbar('Ticks', [3, 4, 5, 6, 7]);
%ylabel(grad, 'Steps to Steady-State')




end